@@ -17,11 +17,12 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
-
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <gst/check/gstcheck.h>
 
@@ -60,7 +61,7 @@ GST_START_TEST (test_registry)
     GstPlugin *plugin = GST_PLUGIN (g->data);
 
     /* one for the registry, one for the list */
-    GST_DEBUG ("refcount %d %s", GST_OBJECT_REFCOUNT_VALUE (plugin),
+    GST_DEBUG ("Plugin refcount %d %s", GST_OBJECT_REFCOUNT_VALUE (plugin),
         gst_plugin_get_name (plugin));
     ASSERT_OBJECT_REFCOUNT (plugin, "plugin in registry", 2);
 
@@ -73,9 +74,8 @@ GST_START_TEST (test_registry)
     GstPluginFeature *feature = GST_PLUGIN_FEATURE (g->data);
 
     /* one for the registry, one for the list */
-    GST_DEBUG ("refcount %d %s", GST_OBJECT_REFCOUNT_VALUE (feature),
+    GST_DEBUG ("Feature refcount %d %s", GST_OBJECT_REFCOUNT_VALUE (feature),
         GST_OBJECT_NAME (feature));
-    ASSERT_OBJECT_REFCOUNT (feature, "feature in registry", 2);
     gst_object_unref (feature);
   }
   g_list_free (list);
